package Tietorakenteet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Hiihdot {
    private ArrayList<Hiihto> hiihdot = new ArrayList<Hiihto>();
    
    /**
     * Muodostaja
     */
    public Hiihdot(){
       //
    }
    
    
    /**
     * @param hiihto suoritus, joka lisätään rekisteriin
     * @example
     * <pre name="test">
     * Hiihdot ht = new Hiihdot();
     * Hiihto h = new Hiihto(0);
     * ht.lisaa(h);
     * ht.getHiihdot() === 1;
     * </pre>
     */
    public void lisaa(Hiihto hiihto){
        hiihdot.add(hiihto);
    }

    
    /**
     * Korvaa hiihdon tietorakenteessa.
     * Etsitään samalla tunnusnumerolla oleva hiihto ja ei löydy niin lisätään uutena.
     * @param h lisättävän hiihdon viite.
     * @throws SailoException jos tietorakenne on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException,CloneNotSupportedException
     * #PACKAGEIMPORT
     * Hiihdot hiihdot = new Hiihdot();
     * Hiihto h1 = new Hiihto(0), h2 = new Hiihto(0);
     * hiihdot.getHiihdot() === 0;
     * hiihdot.korvaaTaiLisaa(h1); hiihdot.getHiihdot() === 1;
     * hiihdot.korvaaTaiLisaa(h2); hiihdot.getHiihdot() === 2;
     * Hiihto h3 = h1.clone();
     * hiihdot.korvaaTaiLisaa(h3); hiihdot.getHiihdot() === 2;
     * </pre>
     */ 
    public void korvaaTaiLisaa(Hiihto h) throws SailoException {
        int id = h.getID();
        for (int i = 0; i < getHiihdot(); i++) {
            if (hiihdot.get(i).getID() == id) {
                hiihdot.set(i, h);
                return;
            }
        }
        lisaa(h);
    }

    
    /**
     * Poistaa kaikki tietyn tietyn jäsenen hiihdot
     * @param tunnusNro viite siihen, mihin liittyvät tietueet poistetaan
     * @return montako poistettiin 
     * @example
     * <pre name="test">
     *  Hiihdot ht = new Hiihdot();
     *  Hiihto h1 = new Hiihto(0);
     *  Hiihto h2 = new Hiihto(0);
     *  ht.lisaa(h1); ht.lisaa(h2);
     *  ht.getHiihdot() === 2;
     *  ht.poistaHiihdot(0) === 2; ht.getHiihdot() === 0;
     * </pre>
     */
    public int poistaHiihdot(int tunnusNro) {
        int n = 0;
        for (Iterator<Hiihto> it = hiihdot.iterator(); it.hasNext();) {
            Hiihto h = it.next();
            if ( h.getJasenID() == tunnusNro ) {
                it.remove();
                n++;
            }
        }
        return n;
    }
    
    
    /**
     * Palauttaa viitteen i:teen hiihtoon.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.anna(0) === eka;
     * hiihdot.anna(1) === toka;
     * </pre>  
     */
    public Hiihto anna(int i) throws IndexOutOfBoundsException {
        
        if (i < 0 || hiihdot.size() <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return hiihdot.get(i);
    }
    
    
    /**
    * Lukee harrastukset tiedostosta.
    * @throws SailoException jos lukeminen epäonnistuu
    * @example
    * <pre name="test">
    * #THROWS SailoException 
    * #import java.io.File;
    *  Hiihdot hiihdot = new Hiihdot();
    *  Hiihto h1 = new Hiihto(1);
    *  Hiihto h2 = new Hiihto(1);
    *  Hiihto h3 = new Hiihto(1); 
    *  Hiihto h4 = new Hiihto(2); 
    *  Hiihto h5 = new Hiihto(2); 
    *  File ftied = new File("hiihdot.dat");
    *  ftied.delete();
    *  hiihdot.lueTiedostosta(); #THROWS SailoException
    *  hiihdot.lisaa(h1);
    *  hiihdot.lisaa(h2);
    *  hiihdot.lisaa(h3);
    *  hiihdot.lisaa(h4);
    *  hiihdot.lisaa(h5);
    *  hiihdot.tallenna();
    *  hiihdot = new Hiihdot();
    *  hiihdot.lueTiedostosta();
    *  hiihdot.getHiihdot() === 5;
    *  hiihdot.lisaa(h1);
    *  hiihdot.tallenna();
    *  hiihdot.getHiihdot() === 6;
    *  ftied.delete() === true;
    *  File fbak = new File("hiihdot.bak");
    *  fbak.delete() === true;
    * </pre>
    */
    public void lueTiedostosta() throws SailoException {
        try ( BufferedReader fi = new BufferedReader(new FileReader("hiihdot.dat")) ) {
            String rivi;
            while ( (rivi = fi.readLine()) != null ) {
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Hiihto h = new Hiihto(0);
                h.parse(rivi); // voisi olla virhekäsittely
                lisaa(h);
            }
        } catch ( FileNotFoundException e ) {
            throw new SailoException("Tiedosto 'hiihdot.dat' ei aukea");
        } catch ( IOException e ) {
            throw new SailoException("Ongelmia tiedoston kanssa: " + e.getMessage());
        }
    }


    /**
     * Tallentaa harrastukset tiedostoon.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void tallenna() throws SailoException {
        File fbak = new File("hiihdot.bak");
        File ftied = new File("hiihdot.dat");
        fbak.delete(); //  if ... System.err.println("Ei voi tuhota");
        ftied.renameTo(fbak); //  if ... System.err.println("Ei voi nimetä");

        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath())) ) {
            for (Hiihto h : hiihdot) {
                fo.println(h.toString());
            }
        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }
    }


    /**
     * @param id jasenen ID
     * @return tietyn jäsenen hiihdot listana
     * @example
     * <pre name="test">
     * Hiihdot hiihdot = new Hiihdot();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kol = new Hiihto(2);
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.lisaa(kol);
     * hiihdot.anna(0) === eka;
     * hiihdot.anna(1) === toka;
     * hiihdot.anna(2) === kol;
     * </pre>
     */
    public ArrayList<Hiihto> getJasenenHiihdot(int id) {
        ArrayList<Hiihto> jasenenHiihdot = new ArrayList<Hiihto>();
        for (int i = 0; i < hiihdot.size(); i++) {
            if (hiihdot.get(i).getJasenID() == id) jasenenHiihdot.add(hiihdot.get(i));
        }
        return jasenenHiihdot;
    }
    
    
    /**
     * @return rekisterin hiihtosuoritusten maara
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(3);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.lisaa(kolmas);
     * hiihdot.getHiihdot() === 3;
     * </pre>
     */
    public int getHiihdot() {
        return hiihdot.size();
    }
    
    
    /**
     * Poistetaan valittu hiihto rekisteristä
     * @param h hiihto joka poistetaan
     * @return true jos poisto onnistui
     * @example
     * <pre name="test">
     * Hiihdot ht = new Hiihdot();
     * Hiihto h1 = new Hiihto(0); Hiihto h2 = new Hiihto(0);
     * ht.lisaa(h1); ht.lisaa(h2);
     * ht.getHiihdot() === 2;
     * ht.poista(h1) === true;
     * ht.getHiihdot() === 1;
     * </pre>
     */
    public boolean poista(Hiihto h) {
        for (int i = 0; i < hiihdot.size(); i++) {
            if (h.getID() == hiihdot.get(i).getID()) hiihdot.remove(hiihdot.get(i)); 
        }
        return true;
    }

    
    /**
     * @param j jäsen jonka kilometrit lasketaan
     * @return kilometrit
     * @example
     * <pre name="test">
     * Jasen j = new Jasen();
     * Hiihdot ht = new Hiihdot();
     * Hiihto h1 = new Hiihto(1);
     * h1.aseta(2, "10");
     * ht.lisaa(h1);
     * ht.laskeKm(j) === "10.0";
     * Hiihto h2 = new Hiihto(1);
     * h2.aseta(2, "10");
     * ht.lisaa(h2);
     * ht.laskeKm(j) === "20.0"; 
     * </pre>
     */
    public String laskeKm(Jasen j) {
        ArrayList<Hiihto> jasenenHiihdot = getJasenenHiihdot(j.getID());
        double km = 0;
        for (Hiihto h : jasenenHiihdot) {
            km += Mjonot.erotaDouble(h.anna(2), 0);
        }
        return Double.toString(km);
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Hiihdot hiihdot = new Hiihdot();
        Hiihto lenkki = new Hiihto(1), toinen = new Hiihto(2);
        try {
            hiihdot.lisaa(lenkki);
            hiihdot.lisaa(toinen);
            System.out.println("========== Hiihdot testi ==============");
            for (int i=0; i < hiihdot.getHiihdot(); i++) {
                Hiihto hiihto = hiihdot.anna(i);
                System.out.println("Hiihto nro: " + i);
                hiihto.tulosta(System.out);
            }
        } catch ( Exception ex ) {
            System.out.println(ex.getMessage());
        }
    }
}