package Tietorakenteet;

import java.io.OutputStream;
import java.io.PrintStream;
import java.time.LocalDate;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author MHo
 * @version 20.2.2020
 * Jasen -luokka, joka lisää jäsenen rekisteriin.
 */
public class Hiihto implements Cloneable, Tietue {
    private int id;
    private int jasenID;
    private String pvm = "";
    private String matka = "";
    private String aika = "";
    private String lampotila = "";
    private String keskisyke = "";
    private String maksimisyke = "";
    private String kalorit = "";
    
    private static int juoksevaNro = 1;
    
    /**
     * Muodostaja
     * @param jasenID jolle hiihtosuoritus lisätään
     */
    public Hiihto(int jasenID){
        id = juoksevaNro;
        this.jasenID = jasenID;
        juoksevaNro++;
    }

    @Override
    public int getKenttia() {
        return 7;
    }

    @Override
    public int ekaKentta() {
        return 1;
    }

    
    /**
     * Hiihto tallennetaan vain, jos sillä on päivämäärä.
     * Samannimisellä metodilla tarkistetaan Jasen -luokassa, että lisättävällä jäsenellä on nimi.
     * Tämän vuoksi palautetaan tässä päivämäärä tarkistustietona.
     */
    @Override
    public String getNimi() {
        return pvm + " ";
    }

    
    /**
     * @example
     */
    @Override
    public int getID() {
        return id;
    }
    
    
    /**
     * Palauttaa k:tta jäsenen kenttää vastaavan kysymyksen
     * @param k kuinka monennen kentän kysymys palautetaan (0-alkuinen)
     * @return k:netta kenttää vastaava kysymys
     * @example
     * <pre name="test">
     * Hiihto h = new Hiihto(0);
     * h.getKysymys(1) === "Päivämäärä";
     * h.getKysymys(2) === "Matka";
     * h.getKysymys(7) === "Kalorit";
     * h.getKysymys(8) === "Virheellinen kenttä";
     * </pre>
     */
    @Override
    public String getKysymys(int k) {
        switch ( k ) {
        case 1: return "Päivämäärä";
        case 2: return "Matka";
        case 3: return "Aika";
        case 4: return "Lämpötila";
        case 5: return "Keskisyke";
        case 6: return "Maksimisyke";
        case 7: return "Kalorit";
        default: return "Virheellinen kenttä";
        }
    }

    
    /**
     * @example
     * <pre name="test">
     * Hiihto h = new Hiihto(0);
     * h.parse("0|1|1.1.2020|10|1:30|-5|180|200|800");
     * h.anna(1) === "1.1.2020";
     * h.anna(7) === "800";
     * h.anna(8) === "Tietoa ei löydy!";
     * </pre>
     */
    @Override
    public String anna(int k) {
        switch ( k ) {
        case 1: return "" + pvm;
        case 2: return "" + matka;
        case 3: return "" + aika;
        case 4: return "" + lampotila;
        case 5: return "" + keskisyke;
        case 6: return "" + maksimisyke;
        case 7: return "" + kalorit;
        default: return "Tietoa ei löydy!";
        }
    }

    
    /**
     * Asettaa k:n kentän arvoksi parametrina tuodun merkkijonon arvon
     * @param k kuinka monennen kentän arvo asetetaan
     * @param jono joka asetetaan kentän arvoksi
     * @return null jos asettaminen onnistuu, muuten vastaava virheilmoitus.
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.aseta(1,"1.5.2020") === null;
     *   h.aseta(1,"1.13.2020") === "Tarkista kuukausi";
     *   h.aseta(1,"1.5.2022") === "Tarkista vuosi";
     *   h.aseta(2,"10") === null;
     *   h.aseta(2,"abc") === "Tarkista matka";
     *   h.aseta(3,"aika") === "Tarkista aika";
     *   h.aseta(5,"syke") === "Tarkista keskisyke";
     *   h.aseta(5,"100") === null;
     *   h.aseta(6,"syke") === "Tarkista maksimisyke";
     *   h.aseta(6,"100") === null;
     *   h.aseta(7,"kal") === "Tarkista kalorit";
     *   h.aseta(7,"100") === null;
     * </pre>
     */
    @Override
    public String aseta(int k, String jono) {
        if (jono == "") return "";
        String tjono = jono.trim();
        StringBuilder sb = new StringBuilder(tjono);
        switch ( k ) {
        case 1:
            if (tjono.equals("")) return null;
            try {
                int tulos = Pvm.pvmParse(sb);
                if (tulos == 1) return "Tarkista päivä";
                if (tulos == 2) return "Tarkista kuukausi";
                if (tulos == 3) return "Tarkista vuosi";
                pvm = tjono;
            } catch ( NumberFormatException ex ) {
                return "Syötä päivämäärä muodossa pp.kk.vvvv" + ex;
            }
                return null;
        case 2:
            try {
                if (Mjonot.erotaDouble(tjono, 0) == 0) return "Tarkista matka";
                matka = Double.toString(Mjonot.erotaDouble(tjono, 0));
            } catch ( NumberFormatException ex ) {
                return "Tarkista matka" + ex.getMessage();
            } return null;
        case 3:
            try {
                int h = Mjonot.erotaInt(sb, -1);
                int m = Mjonot.erotaInt(sb, -1);
                if (h < 0 || m < 0 || m > 59) return "Tarkista aika";
                if (tjono.charAt(2) != ':') return "Syötä aika muodossa hh:mm";
                aika = Integer.toString(h) + ':' + Integer.toString(m);
            } catch ( NumberFormatException ex ) {
                return "Tarkista aika" + ex.getMessage();
            }
                return null;
        case 4:
            if (tjono.equals("")) return null;
            try {
                String etu = "-";
                String luku = "";
                if (tjono.indexOf('+') >= 0) etu = "";
                if (tjono.charAt(0) == '-' || tjono.charAt(0) == '+') luku = tjono.substring(1);
                else luku = tjono.substring(0);
                if (Mjonot.erotaDouble(luku, 100) >= 100) return "Tarkista lämpötila";
                lampotila = etu + luku;
            } catch ( NumberFormatException ex ) {
                return "Tarkista lämpötila" + ex.getMessage();
            }
            return null;
        case 5:
            if (tjono.equals("")) return null;
            try {
                
                double syke = Mjonot.erotaDouble(sb, -1);
                if (syke < 0) return "Tarkista keskisyke";
                keskisyke = Double.toString(syke);
            } catch ( NumberFormatException ex ) {
                return "Tarkista keskisyke" + ex.getMessage();
            }
            return null;
        case 6:
            if (tjono.equals("")) return null;
            try {
                double syke = Mjonot.erotaDouble(sb, -1);
                if (syke < 0) return "Tarkista maksimisyke";
                maksimisyke = Double.toString(syke);
            } catch ( NumberFormatException ex ) {
                return "Tarkista maksimisyke" + ex.getMessage();
            }
            return null;
        case 7:
            if (tjono.equals("")) return null;
            try {
                double kal = Mjonot.erotaDouble(sb, -1);
                if (kal < 0) return "Tarkista kalorit";
                kalorit = Double.toString(kal);
            } catch ( NumberFormatException ex ) {
                return "Tarkista kalorit" + ex.getMessage();
            }
            return null;
        default: return "Virheellinen syöttö!";
        }
    }

    
    /**
     * Kloonataan hiihto
     * @return Object kloonattu hiihto
     * @example
     * <pre name="test">
     * #THROWS CloneNotSupportedException 
     *   Hiihto h = new Hiihto(0);
     *   h.parse("   1.5.2020  |  10.0   | 1:30");
     *   Hiihto kopio = h.clone();
     *   kopio.toString() === h.toString();
     *   h.parse("   1.5.2020  |  20.0   | 1:00");
     *   kopio.toString().equals(h.toString()) === false;
     * </pre>
     */
    @Override
    public Hiihto clone() throws CloneNotSupportedException {
        Hiihto uusi;
        uusi = (Hiihto) super.clone();
        return uusi;
    }
    

    /**
     * Selvittää harrastuksen tiedot | erotellusta merkkijonosta.
     * Pitää huolen että seuraavaNro on suurempi kuin tuleva tunnusnro.
     * @param rivi josta harrastuksen tiedot otetaan
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.parse("1  | 5 |  10.3.2020 | 20 | 1:30 | 150 ");
     *   h.getJasenID() === 5;
     *   h.toString().startsWith("1|5|10.3.2020|20|");
     *   h.parse("1| 20");
     *   h.getJasenID() === 20;
     *   h.toString().startsWith("1|20|");
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        id = Mjonot.erota(sb, '|', id);
        jasenID = Mjonot.erota(sb, '|', jasenID);
        pvm = Mjonot.erota(sb, '|', pvm);
        matka = Mjonot.erota(sb, '|', matka);
        aika = Mjonot.erota(sb, '|', aika);
        lampotila = Mjonot.erota(sb, '|', lampotila);
        keskisyke = Mjonot.erota(sb, '|', keskisyke);
        maksimisyke = Mjonot.erota(sb, '|', maksimisyke);
        kalorit = Mjonot.erota(sb, '|', kalorit);
        juoksevaNro++;
    }
    
    
    /**
     * Palauttaa harrastuksen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return harrastus tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.parse("   1   |  3  |   31.3.2020  | 10 | 1:30| -5|180|200|800 ");
     *   h.toString() === "1|3|31.3.2020|10|1:30|-5|180|200|800";
     * </pre>
     */
    @Override
    public String toString() {
        return id + "|" + jasenID + "|" + pvm + "|" + matka + "|" + aika + "|" + lampotila + "|" + keskisyke + "|"
                + maksimisyke + "|" + kalorit;
    }
    
    
    /**
     * @return hiihdon jasen-ID
     */
    public int getJasenID() {
        return jasenID;
    }
    
    
    /**
     * Haetaan hiihdon kaikki tiedot
     * @return hiihtosuorituksen tiedot merkkijonotaulukkona
     * @example
     * <pre name="test">
     * #import java.util.Arrays;
     * Hiihto eka = new Hiihto(1);
     * eka.parse("1|1|1.1.2020|10|1:30|-5|180|200|800");
     * eka.anna(7) === "800";
     * Arrays.toString(eka.getTiedot()) === "[1.1.2020, 10, 1:30, -5, 180, 200, 800]";
     * </pre>
     */
    public String[] getTiedot() {
        String[] tiedot = new String[] {pvm, matka, aika, lampotila,
                keskisyke, maksimisyke, kalorit};
        return tiedot;
    }
    
    
    /**
     * Tulostetaan hiihdon tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", jasenID, 3) + " " + pvm + " " + matka + "km aika: " + aika
                + " Lämpötila: " + lampotila + "  Keskisyke: " + keskisyke + " Maksimisyke: " + maksimisyke + " Kalorit: " + kalorit);
    }
    
    
    /**
     * Tulostetaan hiihdon tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Hiihto lenkki = new Hiihto(1);
        lenkki.tulosta(System.out);
    }
}