package Tietorakenteet;

import java.io.OutputStream;
import java.io.PrintStream;
import java.time.LocalDate;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author MHo
 * @version 20.2.2020
 * Jasen -luokka, joka lisää jäsenen rekisteriin.
 */
public class Jasen implements Cloneable, Tietue {
    private int id;
    private String eNimi = "";
    private String sNimi = "";
    private String sAika = "";
    private String pKunta = "";
    private String puh = "";
    private String sPosti = "";
    private String lVuosi = "";
    
    private static int juoksevaNro = 1;
    
    /**
     * Muodostaja
     */
    public Jasen(){
        id = juoksevaNro;
        juoksevaNro++;
    }
    
    
    /**
     * Tehdään identtinen klooni jäsenestä
     * @return Object kloonattu jäsen
     * @example
     * <pre name="test">
     * #THROWS CloneNotSupportedException 
     *   Jasen jasen = new Jasen();
     *   jasen.parseJasen("   3  |  Ankka Aku   | 123");
     *   Jasen kopio = jasen.clone();
     *   kopio.toString() === jasen.toString();
     *   jasen.parseJasen("   4  |  Ankka Tupu   | 123");
     *   kopio.toString().equals(jasen.toString()) === false;
     * </pre>
     */
    @Override
    public Jasen clone() throws CloneNotSupportedException {
        Jasen uusi;
        uusi = (Jasen) super.clone();
        return uusi;
    }
    
    
    /**
     * Muodostaja merkkijonosta
     * @param rivi tiedostosta
     * @example
     * <pre name="test">
     * Jasen j = new Jasen();
     * j.parseJasen("  3  |  Ukko   |  Pekka  ");
     * j.toString() === "3|Ukko|Pekka|||||";
     * </pre>
     */
    public void parseJasen(String rivi){
        StringBuilder sb = new StringBuilder(rivi);
        id = Mjonot.erota(sb, '|', juoksevaNro);
        if(id >= juoksevaNro) juoksevaNro = id + 1;
        eNimi = Mjonot.erota(sb, '|', eNimi);
        sNimi = Mjonot.erota(sb, '|', sNimi);
        sAika = Mjonot.erota(sb, '|', sAika);
        pKunta = Mjonot.erota(sb, '|', pKunta);
        puh = Mjonot.erota(sb, '|', puh);
        sPosti = Mjonot.erota(sb, '|', sPosti);
        lVuosi = Mjonot.erota(sb, '|', lVuosi);
    }
    
    
    @Override
    /**
     * Palauttaa jäsenen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return jäsen tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Jasen jasen = new Jasen();
     *   jasen.parseJasen("   3  |  Ankka Aku   | 31.3.2020  | Jykyla | 040123 | aku@posti.fi");
     *   jasen.toString().startsWith("3|Ankka Aku|31.3.2020|Jykyla|") === true; // on enemmän kuin 3 kenttää, siksi loppu |
     * </pre>  
     */
    public String toString() {
        return "" +
                id + "|" + eNimi + "|" + sNimi + "|" + sAika + "|" + pKunta + "|" + puh + "|" + sPosti + "|" + lVuosi;
    }
    
    
    /**
    * Antaa k:n kentän sisällön merkkijonona
    * @param k monenenko kentän sisältö palautetaan
    * @return kentän sisältö merkkijonona
    * @example
    * <pre name="test">
    * Jasen j = new Jasen();
    * j.parseJasen("1|Ukko|Pekka");
    * j.anna(1) === "Ukko";
    * j.anna(2) === "Pekka";
    * </pre>
    */
    @Override
    public String anna(int k) {
    switch ( k ) {
        case 1: return "" + eNimi;
        case 2: return "" + sNimi;
        case 3: return "" + sAika;
        case 4: return "" + pKunta;
        case 5: return "" + puh;
        case 6: return "" + sPosti;
        case 7: return "" + lVuosi;
        default: return "Tietoa ei löydy!";
        }
    }
    
    
    /**
     * Asettaa k:n kentän arvoksi parametrina tuodun merkkijonon arvon
     * @param k kuinka monennen kentän arvo asetetaan
     * @param jono joka asetetaan kentän arvoksi
     * @return null jos asettaminen onnistuu, muuten vastaava virheilmoitus.
     * @example
     * <pre name="test">
     *   Jasen jasen = new Jasen();
     *   jasen.aseta(1,"harri") === null;
     *   jasen.aseta(2,"olli") === null;
     *   jasen.aseta(3,"01.10.1900") === null;
     *   jasen.aseta(3,"01.22.1900") === "Tarkista kuukausi";
     *   jasen.aseta(3,"01.10.800") === "Tarkista vuosi";
     *   jasen.aseta(4,"akaa") === null;
     *   jasen.aseta(4,"01") === "Paikkakunta ei saa sisältää numeroita";
     *   jasen.aseta(5,"12345678900") === "Tarkista puhelinnumero";
     *   jasen.aseta(5,"1234567890") === null;
     *   jasen.aseta(6,"kissa") === "@-merkki puuttuu!"
     *   jasen.aseta(7,"2000") === null;
     *   jasen.aseta(7,"00") === "Tarkista liittymisvuosi";
     * </pre>
     */
    @Override
    public String aseta(int k, String jono) {
        String tjono = jono.trim();
        StringBuilder sb = new StringBuilder(tjono);
        switch ( k ) {
        case 1:
            if (tjono.equals("")) return null;
            String e1 = tjono.substring(0, 1).toUpperCase();
            eNimi = e1 + tjono.substring(1);
            return null;
        case 2:
            if (tjono.equals("")) return null;
            String s1 = tjono.substring(0, 1).toUpperCase();
            sNimi = s1 + tjono.substring(1);;
            return null;
        case 3:
            if (tjono.equals("")) return null;
            try {
                int tulos = Pvm.pvmParse(sb);
                if (tulos == 1) return "Tarkista päivä";
                if (tulos == 2) return "Tarkista kuukausi";
                if (tulos == 3) return "Tarkista vuosi";
                sAika = tjono;
            } catch ( NumberFormatException ex ) {
                return "Syötä syntymäaika muodossa pp.kk.vvvv" + ex;
            }
                return null;
        case 4:
            if (tjono.equals("")) return null;
            if (Mjonot.erotaDouble(sb, 0) != 0) return "Paikkakunta ei saa sisältää numeroita";
            String p1 = tjono.substring(0, 1).toUpperCase();
            pKunta = p1 + tjono.substring(1);
            return null;
        case 5:
            try {
                if (tjono.equals("")) return null;
                if (tjono.length() > 10) return "Tarkista puhelinnumero";
                    puh = Mjonot.erotaEx(sb, '§', puh);
            }   catch ( NumberFormatException ex ) {
                return "Tarkista puhelinnumero" + ex;
            }
            return null;
        case 6:
            try {
                if (tjono.equals("")) return null;
                if (tjono.indexOf("@") < 0) return "@-merkki puuttuu!";
                sPosti = tjono;
            } catch ( NumberFormatException ex ) {
                    return "Tarkista sähköposti" + ex;
            }
            return null;
        case 7:
            if (tjono.equals("")) return null;
            try {
                int a = Mjonot.erotaInt(tjono, 0);
                if (a < 1900 || a > Calendar.getInstance().get(Calendar.YEAR)) return "Tarkista liittymisvuosi";
                lVuosi = Integer.toString(a);
            } catch ( NumberFormatException ex ) {
                return "Tarkista liittymisvuosi" + ex;
                }
            return null;
        default: return "Virheellinen syöttö!";
        }
    }
    

    /**
     * @return jäsenen tietojen määrän
     */
    @Override
    public int getKenttia() {
        return 7;
    }
    
    
    /**
     * @return ensimmäisen syötettävän kentän indeksi
     */
    @Override
    public int ekaKentta() {
        return 1;
    }
    
    
    /**
     * Palauttaa k:tta jäsenen kenttää vastaavan kysymyksen
     * @param k kuinka monennen kentän kysymys palautetaan (0-alkuinen)
     * @return k:netta kenttää vastaava kysymys
     * @example
     * <pre name="test">
     * Jasen eka = new Jasen();
     * eka.getKysymys(0) === "Virheellinen kenttä";
     * eka.getKysymys(2) === "Sukunimi";
     * eka.getKysymys(8) === "Virheellinen kenttä";
     * </pre>
     */
    @Override
    public String getKysymys(int k) {
        switch ( k ) {
        case 1: return "Etunimi";
        case 2: return "Sukunimi";
        case 3: return "Syntymäaika";
        case 4: return "Paikkakunta";
        case 5: return "Puhelinnumero";
        case 6: return "Sähköposti";
        case 7: return "Liittymisvuosi";
        default: return "Virheellinen kenttä";
        }
    }
    
    
    /** 
     * Jäsenten vertailija 
     */ 
    public static class Vertailija implements Comparator<Jasen> { 
        private int k;  
         
        @SuppressWarnings("javadoc") 
        public Vertailija(int k) { 
            this.k = k; 
        } 
         
        @Override 
        public int compare(Jasen jasen1, Jasen jasen2) { 
            return jasen1.getKysymys(k).compareToIgnoreCase(jasen2.getKysymys(k)); 
        } 
    }
    
    
    /**
     * @return jäsenen ID-numero
     * @example
     * <pre name="test">
     * Jasen j = new Jasen();
     * j.parseJasen("1|Ukko");
     * j.getID() === 1;
     * </pre>
     */
    @Override
    public int getID() {
        return id;
    }
    
    
    /**
     * @return jäsenen nimi
     * <pre name="test">
     * Jasen j = new Jasen();
     * j.parseJasen("1|Ukko|Pekka");
     * j.getNimi() === "Ukko Pekka";
     * </pre>
     */
    @Override
    public String getNimi() {
        return eNimi + " " + sNimi;
    }
    
    
    /**
     * @return paikkakunta
    public String tarkista() {
        return pKunta;
    }
    */
    
    
    /**
     * Haetaan henkilön tiedot
     * @return jasenen tiedot merkkijono -taulukkona
     * @example
     * #import java.util.Arrays;
     * Jasen testi = new Jasen();
     * testi.parseJasen("Ukko|Pekka|1.1.2000|Akaa|040|@mail|2020");
     * Arrays.toString(testi.getTiedot()) === "[Ukko, Pekka, 1.1.2000, Akaa, 040, @mail, 2020]";
    public String[] getTiedot() {
        String[] tiedot = new String[] {eNimi, sNimi, sAika, pKunta, puh, sPosti, lVuosi};
        return tiedot;
    }
    */
    
    
    /**
     * Tulostetaan henkilön tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", id, 3) + ' ' + eNimi + ' ' + sNimi);
        out.println(sAika + ' ' + pKunta + ' ' + puh + ' ' + sPosti);
        out.print("Liittynyt " + lVuosi + ".");
    }
    
    
    /**
     * Tulostetaan henkilön tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }

    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Jasen nokka = new Jasen();
        nokka.tulosta(System.out);
    }
}