package Tietorakenteet;

import java.util.Calendar;

import fi.jyu.mit.ohj2.*;

/**
 * Alustava luokka päivämäärää varten
 * @author MHo
 * @version 1.0, 07.02.2003
 * @version 1.1, 14.02.2003
 * @version 1.2, 17.02.2003
 * @version 1.3, 11.02.2008
 */
public class Pvm {

    private int pv;
    private int kk;
    private int vv;


    /**
     *  Muuttaa päivämäärän nykypäivälle.
     *  Todo: pitää vaihtaa hakemaan päivämäärä oikeasti. 
     *  Mutta perinnässäkään ei nyt saa käyttää enempää attribuutteja kuin tässä on
     */
    public void paivays() {
        pv = Calendar.getInstance().get(Calendar.DATE);;
        kk = Calendar.getInstance().get(Calendar.MONTH);;
        vv = Calendar.getInstance().get(Calendar.YEAR);;
    }


    /**
     * Alustetaan päivämäärä. 0-arvot eivät muuta vastaavaa attribuuttia
     * @param ipv päivän alustus
     * @param ikk kuukauden alustus
     * @param ivv vuoden alustus
     * @return true jos päivämäärä okein, muuten false
     * @example
     * <pre name="test">
     * Pvm pvm = new Pvm(12, 2, 2020);
     * pvm.toString() === "12.2.2020";
     * pvm.alusta(12, 13, 2020);
     * pvm.toString() === "12.2.2020";
     * pvm.alusta(12, 0, 2020);
     * pvm.toString() === "12.2.2020";
     * </pre>
     */
    public static int alusta(int ipv, int ikk, int ivv) {
        int p = ipv; int k = ikk; int v = ivv;
        
        if (p == 0) return 1;                 //ipv = Calendar.getInstance().get(Calendar.DATE);;
        if (k == 0 || k > 12 ) return 2;      //ikk = Calendar.getInstance().get(Calendar.MONTH);;
        if (v < 1900 || v > Calendar.getInstance().get(Calendar.YEAR)) return 3;
        //v = Calendar.getInstance().get(Calendar.YEAR);;
        if (p > KPITUUDET[karkausvuosi(ivv)][ikk-1]) return 1;
        return 0;
        
        /**
        if ( ipv > 0 ) this.pv = ipv;
        if ( ikk > 0 ) this.kk = ikk;
        if ( ivv > 0 ) this.vv = ivv;
        if ( this.vv < 50 ) this.vv += 2000;
        if ( this.vv < 100 ) this.vv += 1900;
        */
    }


    public static final int KPITUUDET[][] = {
            // 1  2   3   4   5   6   7   8   9   10  11  12
            { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
            { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
    };


    /**
     * Palautetaan tieto siitä onko tutkittava vuosi karkausvuosi vai ei
     * @param vv tutkittava vuosi
     * @return 1 jos on karkausvuosi ja 0 jos ei ole
     * @example
     * <pre name="test">
     *   karkausvuosi(1900) === 0
     *   karkausvuosi(1900) === 0
     *   karkausvuosi(1901) === 0
     *   karkausvuosi(1996) === 1
     *   karkausvuosi(2000) === 1
     *   karkausvuosi(2001) === 0
     *   karkausvuosi(2004) === 1
     * </pre>
     */
    public static int karkausvuosi(int vv) {
        if ( vv % 400 == 0 ) return 1;
        if ( vv % 100 == 0 ) return 0;
        if ( vv % 4 == 0 ) return 1;
        return 0;
    }
    

    /** Alustetaan kaikki attribuutit oletusarvoon
    public Pvm() {
        this(0, 0, 0);
    }


    /** 
     * Alustetaan kuukausi ja vuosi oletusarvoon
     * @param pv päivän alustusarvo
     
    public Pvm(int pv) {
        this(pv, 0, 0);
    }


    /**
     * Alustetaan vuosi oletusarvoon
     * @param pv päivän alustusarvo
     * @param kk kuukauden oletusarvo
     
    public Pvm(int pv, int kk) {
        this(pv, kk, 0);
    }


    /**
     * Alustetaan vuosi oletusarvoon
     * @param pv päivän alustusarvo
     * @param kk kuukauden oletusarvo
     * @param vv vuoden alustusarvo
     
    public Pvm(int pv, int kk, int vv) {
        paivays();
        alusta(pv, kk, vv);
    }
    


    /**
     * Muodostaja. Alustetaan päivämäärä merkkijonosta
     * @param s muotoa 12.3.2008 oleva merkkijono
    public Pvm(String s) {
        paivays();
        pvmParse(s);
    } 
    */

    /**
     * Ottaa päivämäärän tiedot merkkijonosta joka on muotoa 17.2.2007
     * Jos joku osa puuttuu, sille käytetään tämän päivän arvoa oletuksena.
     * @param sb tutkittava merkkijono
     * @return numeroilmoituksen siitä, onko päivämäärä oikein
     */
    protected static int pvmParse(StringBuilder sb) {
        int p = Mjonot.erota(sb, '.', 0);
        int k = Mjonot.erota(sb, '.', 0);
        int v = Mjonot.erota(sb, ' ', 0);
        return alusta(p, k, v);
        // tai alusta(Mjonot.erota(sb,'.',0),Mjonot.erota(sb,'.',0),Mjonot.erota(sb,'.',0));
    }


    /**
     * Ottaa päivämäärän tiedot merkkijonosta joka on muotoa 17.2.2007
     * Jos joku osa puuttuu, sille käytetään tämän päivän arvoa oletuksena.
     * @param s tutkittava merkkijono
     */
    protected final void pvmParse(String s) {
        pvmParse(new StringBuilder(s));
    }


    /**
     * Ottaa päivämäärän tiedot merkkijonosta joka on muotoa 17.2.2007
     * Jos joku osa puuttuu, sille käytetään tämän päivän arvoa oletuksena.
     * @param s tutkittava merkkijono
     * 
     * @example
     * <pre name="test">
     * Pvm pvm = new Pvm(11,3,2003);
     * pvm.parse("12");           pvm.toString() === "12.3.2003";
     * pvm.parse("..2001");       pvm.toString() === "12.3.2001";
     * pvm.parse("..2009 14:30"); pvm.toString() === "12.3.2009"; 
     * </pre>
     */
    public void parse(String s) {
        pvmParse(s);
    }


    /**
     * Ottaa päivämäärän tiedot merkkijonosta joka on muotoa 17.2.2007
     * Jos joku osa puuttuu, sille käytetään tämän päivän arvoa oletuksena.
     * Merkkijonosta otetaan pois vain se osa, jota tarvitaan.
     * @param sb tutkittava merkkijono
     * 
     * @example
     * <pre name="test">
     * Pvm pvm = new Pvm(11,3,2003);
     * StringBuilder jono = new StringBuilder("12");
     * pvm.parse(jono); pvm.toString() === "12.3.2003"; jono.toString() === "";
     * jono = new StringBuilder("..2001");
     * pvm.parse(jono); pvm.toString() === "12.3.2001"; jono.toString() === "";
     * jono = new StringBuilder("..2009 14:30");
     * pvm.parse(jono); pvm.toString() === "12.3.2009"; jono.toString() === "14:30";
     * </pre>
     */
    public void parse(StringBuilder sb) {
        pvmParse(sb);
    }
    
    
    /**
     * Päivämäärä merkkijonona
     * @return päivämäärä muodossa 17.2.2007
     * @example
     * <pre name="test">
     *   Pvm tammi2011 = new Pvm(1,1,2011);
     *   tammi2011.toString() === "1.1.2011"
     *   Pvm helmi2011 = new Pvm(1,2,2011);
     *   helmi2011.toString() === "1.2.2011"
     *   Pvm tanaan = new Pvm(14,2,2011);
     *   tanaan.toString()    === "14.2.2011"
     *   Pvm maalis97 = new Pvm(1,3,1997);
     *   maalis97.toString()  === "1.3.1997"
     * </pre>
     */
    @Override
    public String toString() {
        return pv + "." + kk + "." + vv;
    }


    /**
     * @return päivän arvo
     * @example
     * <pre name="test">
     *   Pvm pv = new Pvm("14.2.2011");
     *   pv.getPv() === 14;
     * </pre>
     */
    public int getPv() {
        return pv;
    }


    /**
     * @return kuukauden arvo
     * @example
     * <pre name="test">
     *   Pvm pv = new Pvm("14.2.2011");
     *   pv.getKk() === 2;
     * </pre>
     */
    public int getKk() {
        return kk;
    }


    /**
     * @return vuoden arvo
     * @example
     * <pre name="test">
     *   Pvm pv = new Pvm("14.2.2011");
     *   pv.getVv() === 2011;
     * </pre>
     */
    public int getVv() {
        return vv;
    }


    /**
     * Testataan päivämäärä-luokkaa
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Pvm pvm = new Pvm();

        pvm.parse("12.1.1995");
        System.out.println(pvm);
        pvm.parse("15.3");
        System.out.println(pvm);
        pvm.parse("14");
        System.out.println(pvm.getPv());

    }
    
    
     /**
     * @param pv1 1. päivämäärä
     * @param pv2 2. päivämäärä
     * @return -1 jos pv1 ennen pv2
     * return 0 jos pv:t ovat samat
     * return 1 muissa tapauksissa
     * @example
     * <pre name="test">
     * Pvm pv1 = new Pvm("12.2.2020");
     * Pvm pv2 = new Pvm("12.2.2020");
     * compareTo(pv1, pv2) === 0;
     * pv1 = new Pvm("12.2.2020");
     * pv2 = new Pvm("13.2.2020");
     * compareTo(pv1, pv2) === -1;
     * pv1 = new Pvm("12.2.2020");
     * pv2 = new Pvm("12.3.2020");
     * compareTo(pv1, pv2) === -1;
     * pv1 = new Pvm("12.2.2020");
     * pv2 = new Pvm("12.2.2021");
     * compareTo(pv1, pv2) === -1;
     * </pre>
     * */
    public static int compareTo(Pvm pv1, Pvm pv2) {
         if (pv1.toString().equals(pv2.toString())) return 0;
        if (pv1.getVv() < pv2.getVv()
            || (pv1.getVv() == pv2.getVv() && pv1.getKk() < pv2.getKk())
            || (pv1.getKk() == pv2.getKk() && pv1.getPv() < pv2.getPv())
            )return -1;
        return 1;
    }
    
    
    /**
     * Metodi päivämäärien vertailuun
     * @param pv2 Pvm johon verrataan
     * @return kumpi Pvm on pienempi
     * @example
     * <pre name="test">
     * Pvm pv1 = new Pvm("12.2.2020");
     * Pvm pv2 = new Pvm("12.2.2020");
     * pv1.compareTo(pv2) === 0;
     * pv2 = new Pvm("13.2.2020");
     * pv1.compareTo(pv2) === -1;
     * pv2 = new Pvm("12.3.2020");
     * pv1.compareTo(pv2) === -1;
     * pv2 = new Pvm("12.2.2021");
     * pv1.compareTo(pv2) === -1;
     * pv2 = new Pvm("11.2.2020");
     * pv1.compareTo(pv2) === 1;
     * </pre>
     */
    public int compareTo(Pvm pv2) {
        return compareTo(this, pv2);
    }
    
    
    /**
     * Metodilla selvitetään, ovatko päivämäärät samat
     * @param pv2 Pvm johon verrataan
     * @return kumpi Pvm on pienempi
     * @example
     * <pre name="test">
     * Pvm pv1 = new Pvm("12.2.2020");
     * Pvm pv2 = new Pvm("12.2.2020");
     * pv1.equals(pv2) === true;
     * pv2 = new Pvm("13.2.2020");
     * pv1.equals(pv2) === false;
     * pv2 = new Pvm("12.3.2020");
     * pv1.equals(pv2) === false;
     * pv2 = new Pvm("12.2.2021");
     * pv1.equals(pv2) === false;
     * pv2 = new Pvm("11.2.2020");
     * pv1.equals(pv2) === false;
     * </pre>
     */
    public boolean equals(Pvm pv2) {
        if (compareTo(this, pv2) == 0) return true;
        return false;
    }
}