package Tietorakenteet;

import java.util.ArrayList;
import java.util.Collection;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Rekisteri {
    private Jasenet jasenet = new Jasenet();
    private Hiihdot hiihdot = new Hiihdot();
    
    
    /**
     * Muodostaja
     */
    public Rekisteri(){
        //
    }
    
    
    /**
     * Lisää jäsenen taulukkoon.
     * @param jasen lisättävän jäsenen viite
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * rekisteri.getLkm() === 2;
     * </pre>
     */
    public void lisaa(Jasen jasen){
        jasenet.lisaa(jasen);
    }
    
    
    /** 
     * Korvaa jäsenen tietorakenteessa.  Ottaa jäsenen omistukseensa. 
     * Etsitään samalla tunnusnumerolla oleva jäsen.  Jos ei löydy, 
     * niin lisätään uutena jäsenenä. 
     * @param jasen lisätäävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi 
     * @throws SailoException jos tietorakenne on jo täynnä 
     * <pre name="test">
     * #THROWS SailoException,CloneNotSupportedException
     * #PACKAGEIMPORT
     * Rekisteri R = new Rekisteri();
     * Jasen eka = new Jasen(), toka = new Jasen();
     * R.getLkm() === 0;
     * R.korvaaTaiLisaa(eka); R.getLkm() === 1;
     * R.korvaaTaiLisaa(toka); R.getLkm() === 2;
     * Jasen kol = toka.clone();
     * R.korvaaTaiLisaa(kol); R.getLkm() === 2;
     * </pre>
     */ 
    public void korvaaTaiLisaa(Jasen jasen) throws SailoException { 
        jasenet.korvaaTaiLisaa(jasen); 
    }
    
    
    /** 
     * Korvaa hiihdon tietorakenteessa. 
     * Etsitään samalla id:llä oleva hiihto tai lisätään uutena hiihtona.
     * @param h lisättävän hiihto. 
     * @throws SailoException jos tietorakenne on jo täynnä 
     */ 
    public void korvaaTaiLisaa(Hiihto h) throws SailoException { 
        hiihdot.korvaaTaiLisaa(h); 
    } 

    
    
    /**
     * @param i monennenko jäsenen viite haetaan
     * @return viite jäseneen, jonka indeksi on i
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * rekisteri.annaJasen(0) === eka;
     * </pre>
     */
    public Jasen annaJasen(int i) {
        return jasenet.anna(i);
    }
    
    
    /** 
     * Palauttaa "taulukossa" hakuehtoon vastaavien jäsenten viitteet 
     * @param hakuehto hakuehto  
     * @param k etsittävän kentän indeksi  
     * @return tietorakenteen löytyneistä jäsenistä 
     * @throws SailoException Jos jotakin menee väärin
     * @example 
     * <pre name="test">
     *   #THROWS CloneNotSupportedException, SailoException
     *   Rekisteri R = new Rekisteri();
     *   Jasen eka = new Jasen();
     *   eka.aseta(1,"Pekka");
     *   R.lisaa(eka);
     *   Collection<Jasen> loytyneet = R.etsi("*Pe*",1);
     *   loytyneet.size() === 1;
     *   Iterator<Jasen> it = loytyneet.iterator();
     *   it.next() == eka === true; 
     * </pre>
     */ 
    public Collection<Jasen> etsi(String hakuehto, int k) throws SailoException { 
        return jasenet.etsi(hakuehto, k);
    } 
    
    
    /**
     * Lisää hiihto listaan.
     * @param hiihto lisättävän hiihdon viite
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.getHiihdot() === 2;
     * </pre>
     */
    public void lisaaHiihto(Hiihto hiihto){
        hiihdot.lisaa(hiihto);
    }
    
    
    /**
     * @param i monennenko jäsenen viite haetaan
     * @return viite jäseneen, jonka indeksi on i
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.annaHiihto(0) === eka;
     * </pre>
     */
    public Hiihto annaHiihto(int i) {
        return hiihdot.anna(i);
    }
    
    
    /**
     * @param id hiihtajan ID
     * @return halutun jäsenen kaikki hiihdot
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * Rekisteri rekisteri = new Rekisteri();
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.lisaaHiihto(kolmas);
     * </pre>
    */
    public ArrayList<Hiihto> getJasenenHiihdot(int id) {
        return hiihdot.getJasenenHiihdot(id);
    }
    
    
    /**
     * Lukee rekisterin tiedot tiedostosta
     * @throws SailoException jos lukeminen epäonnistuu
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.*;
     * #import java.util.*;
     * Rekisteri r = new Rekisteri();
     *  Jasen j1 = new Jasen(); Jasen j2 = new Jasen();
     *  Hiihto h1 = new Hiihto(1);
     *  Hiihto h2 = new Hiihto(1);
     *  Hiihto h3 = new Hiihto(2);
     *  Hiihto h4 = new Hiihto(2);
     *  File fjas  = new File("nimet.dat");
     *  File fhiih = new File("hiihdot.dat");  
     *  fjas.delete();
     *  fhiih.delete();
     *  r.lueTiedostosta(); #THROWS SailoException
     *  r.lisaa(j1);
     *  r.lisaa(j2);
     *  r.lisaaHiihto(h1);
     *  r.lisaaHiihto(h2);
     *  r.lisaaHiihto(h3);
     *  r.lisaaHiihto(h4);
     *  r.tallenna();
     *  r = new Rekisteri();
     *  r.lueTiedostosta();
     *  r.lisaa(j2);
     *  r.lisaaHiihto(h4);
     *  r.tallenna();
     *  fjas.delete()  === true;
     *  fhiih.delete() === true;
     *  File fjasbak = new File("nimet.bak");
     *  File fhbak = new File("hiihdot.bak");
     *  fjasbak.delete() === true;
     *  fhbak.delete() === true;
     * </pre>
     */

    public void lueTiedostosta() throws SailoException {
        jasenet = new Jasenet();
        hiihdot = new Hiihdot();
        jasenet.lueTiedostosta();
        hiihdot.lueTiedostosta();
    }

        
    /**
     * Tallettaa kerhon tiedot tiedostoon
     * Tallenttaa kerhon tiedot tiedostoon.  
     * Vaikka jäsenten tallettamien epäonistuisi, niin yritetään silti tallettaa
     * harrastuksia ennen poikkeuksen heittämistä.
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            jasenet.tallenna();
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            hiihdot.tallenna();
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }

    
    /**
     * @return jäsenistön lkm
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * rekisteri.lisaa(eka);
     * rekisteri.lisaa(toka);
     * </pre>
     */
    public int getLkm() {
        return jasenet.getLkm();
    }
    
    
    /**
     * @return rekisterin hiihtosuoritusten lukumäärä
     * @example
     * <pre name="test">
     * Rekisteri rekisteri = new Rekisteri();
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * rekisteri.lisaaHiihto(eka);
     * rekisteri.lisaaHiihto(toka);
     * rekisteri.lisaaHiihto(kolmas);
     * rekisteri.getHiihdot() === 3;
     * </pre>
     */
    public int getHiihdot() {
        return hiihdot.getHiihdot();
    }
    
    /**
     * @param j jäsen jonka kilometrit lasketaan
     * @return jäsenen hiihtojen yhteiskilometrit
     * @example
     * <pre name="test">
     * Rekisteri R = new Rekisteri();
     * Jasen j = new Jasen();
     * R.lisaa(j);
     * Hiihto h1 = new Hiihto(1);
     * h1.aseta(2, "10");
     * R.lisaaHiihto(h1);
     * R.laskeKm(j) === "10.0";
     * Hiihto h2 = new Hiihto(1);
     * h2.aseta(2, "10");
     * R.lisaaHiihto(h2);
     * R.laskeKm(j) === "20.0"; 
     * </pre>
     */
    public String laskeKm(Jasen j) {
        return hiihdot.laskeKm(j);
    }
    
    /**
     * Poistaa hiihtäjän ja kaikki hänen hiihtonsa 
     * @param jasen hiihtäjä jokapoistetaan
     * @return montako jäsentä poistettiin
     * @example
     * <pre name="test">
     * #THROWS Exception
     *   Rekisteri rekisteri = new Rekisteri();
     *   Jasen eka = new Jasen();
     *   rekisteri.lisaa(eka);
     *   rekisteri.getLkm() === 1;
     *   rekisteri.poistaHiihtaja(eka) === 1;
     *   rekisteri.getLkm() === 0;
     * </pre>
     */
    public int poistaHiihtaja(Jasen jasen) {
        if ( jasen == null ) return 0;
        int ret = jasenet.poista(jasen.getID()); 
        hiihdot.poistaHiihdot(jasen.getID()); 
        return ret; 
    }
    
    
    /**
     * @param h hiihto joka poistetaan
     * @return true jos poisto onnistui
     * @example
     * <pre name="test">
     * Rekisteri R = new Rekisteri();
     * Hiihto h1 = new Hiihto(0); Hiihto h2 = new Hiihto(0);
     * R.lisaaHiihto(h1); R.lisaaHiihto(h2);
     * R.getHiihdot() === 2;
     * R.poistaHiihto(h1) === true;
     * R.getHiihdot() === 1;
     * </pre>
     */
    public boolean poistaHiihto(Hiihto h) {
        return hiihdot.poista(h);
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Rekisteri rekisteri = new Rekisteri();
        
        try {
            // kerho.lueTiedostosta("kelmit");

            Jasen nokkaraka = new Jasen(), mietaa = new Jasen();
            

            rekisteri.lisaa(nokkaraka);
            rekisteri.lisaa(mietaa);

            System.out.println("============= Kerhon testi =================");

            for (int i = 0; i < rekisteri.getLkm(); i++) {
                Jasen jasen = rekisteri.annaJasen(i);
                System.out.println("Jäsen paikassa: " + i);
                jasen.tulosta(System.out);
                System.out.println();
                System.out.println();
            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }
}