package Tietorakenteet;


/**
 * Rajapinta tietueelle johon voidaan taulukon avulla rakentaa 
 * "attribuutit".
 * @author vesal
 * @version Mar 23, 2012
 * @example
 */
public interface Tietue {

    
    /**
     * @return tietueen kenttien lukumäärä
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.getKenttia() === 7;
     * </pre>
     */
    public abstract int getKenttia();


    
    /**
     * @return ensimmäinen käyttäjän syötettävän kentän indeksi
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.ekaKentta() === 1;
     * </pre>
     */
    public abstract int ekaKentta();


    /**
     * @param k minkä kentän kysymys halutaan
     * @return valitun kentän kysymysteksti
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.getKysymys(1) === "Päivämäärä";
     * </pre>
     */
    public abstract String getKysymys(int k);


    /**
     * @param k Minkä kentän sisältö halutaan
     * @return valitun kentän sisältö
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.parse("1|1|  1.5.2020   |  10.0  |   1:30  | -5 | 180 | 200 | 200  ");
     *   h.anna(1) === "1.5.2020";   
     *   h.anna(2) === "10.0";   
     *   h.anna(3) === "1:30";   
     *   h.anna(5) === "180";   
     * </pre>
     */
    public abstract String anna(int k);

    
    /**
     * Asetetaan valitun kentän sisältö.  Mikäli asettaminen onnistuu,
     * palautetaan null, muutoin virheteksti.
     * @param k minkä kentän sisältö asetetaan
     * @param s asetettava sisältö merkkijonona
     * @return null jos ok, muuten virheteksti
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.aseta(3,"1:30")  === null;
     *   h.aseta(4,"käpy") === null;
     * </pre>
     */
    public abstract String aseta(int k, String s);


    /**
     * Tehdään identtinen klooni tietueesta
     * @return kloonattu tietue
     * @throws CloneNotSupportedException jos kloonausta ei tueta
     * @example
     * <pre name="test">
     * #THROWS CloneNotSupportedException
     *   Hiihto h = new Hiihto(0);
     *   h.parse("   1.5.2020   |  10.0  |   1:30  | 180 | 200 ");
     *   Object kopio = h.clone();
     *   kopio.toString() === h.toString();
     *   h.parse("   1.5.2020   |  2.0  |   0:30  | 120 | 130 ");
     *   kopio.toString().equals(h.toString()) === false;
     *   kopio instanceof Hiihto === true;
     * </pre>
     */
    public abstract Tietue clone() throws CloneNotSupportedException;


    /**
     * Palauttaa tietueen tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return tietue tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Hiihto h = new Hiihto(0);
     *   h.parse("1| 0|   1.5.2020   |  10.0  |   1:30  | -5 | 100 | 120 | 200 ");
     *   h.toString() === "1|0|1.5.2020|10.0|1:30|-5|100|120|200";
     * </pre>
     */
    @Override
    public abstract String toString();

    /**
     * @return hiihtäjän nimi
     */
    public abstract String getNimi();
    
    
    /**
     * @return hiihtäjän ID
     */
    public abstract int getID();
}