package fxHiihtorekisteri;

import java.net.URL;
import java.util.ArrayList;
import java.util.ResourceBundle;

import Tietorakenteet.Rekisteri;
import Tietorakenteet.SailoException;
import Tietorakenteet.Hiihto;
import Tietorakenteet.Tietue;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import fi.jyu.mit.fxgui.StringGrid;
import fi.jyu.mit.ohj2.Mjonot;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.GridPane;
import javafx.stage.Stage;


/**
 * Kysytään tietueen tiedot
 * @author MHo
 * @version 13.1.2020
 * @param <TYPE> Tietueen tyyppi
 *
 */
    public class TietueGUIController<TYPE extends Tietue> implements ModalControllerInterface<TYPE>,Initializable {

    @FXML private Button nappiTallenna;
    @FXML private Button nappiPeruuta;
    @FXML private Button nappiPoistaHiihto;
    @FXML private GridPane gridTietue;
    @FXML private Label labelVirhe;
    @FXML private StringGrid<Hiihto> gridHiihdot;
    
    private TextField[] kentat;
    private TYPE tietueKohdalla;
    private Rekisteri rekisteri;
    
    @FXML void handleTallenna() {
        if ( tietueKohdalla != null && !labelVirhe.getText().equals("")) return;
        if (tietueKohdalla.getNimi().equals(" ")) {
            Dialogs.showMessageDialog("Anna etu- tai sukunimi");
            return;
        }
        if (kentat[0].getText().equals("") || kentat[1].getText().equals("")) {
            Dialogs.showMessageDialog("Anna päivämäärä ja matka");
            return;
        }
        ModalController.closeStage(nappiTallenna);
    }
    
    @FXML void handlePeruuta() {
        tietueKohdalla = null;
        ModalController.closeStage(nappiPeruuta);
    }
    
    @FXML void handlePoistaHiihto() {
        if (poistaHiihto() == true) {
            naytaHiihdot();
            Dialogs.showMessageDialog("Valitsemasi hiihto on poistettu");
        }
        else Dialogs.showMessageDialog("Hiihtoa ei valittu");
        
    }
    
    @Override
    public void initialize(URL arg0, ResourceBundle arg1) {
        // TODO Auto-generated method stub
    }
    
    @Override
    public void setDefault(TYPE oletus) {
        tietueKohdalla = oletus;
    }
    
    @Override
    public TYPE getResult() {
        return tietueKohdalla;
    }

    @Override
    public void handleShown() {
        //
    }
    
    
    /**
     * Tekee tarvittavat alustukset
     */
    protected void alusta() {
       kentat = luoKentat(gridTietue, tietueKohdalla);
       //tyhjenna(kentat);
        for (TextField kentta : kentat) {
            if ( kentta != null)
                kentta.setOnKeyReleased( e -> kasitteleMuutosTietueeseen((TextField)(e.getSource())));
        }
    }
    
    
     /** Tyhjennetään tekstikentät
     * @param kentat taulukko tyhjennettävistä teksikentistä
     */
    public static void tyhjenna(TextField[] kentat) {
        for (TextField kentta : kentat) if (kentta != null) kentta.setText("");
    }
    
    
    /**
     * Luodaan GridPaneen tietueen tiedot
     * @param gridTietue mihin tiedot luodaan
     * @param tietue tietue, jonka tiedoilla luodaan kentät
     * @return luodut tekstikentät
     */
    public static<TYPE extends Tietue> TextField[] luoKentat(GridPane gridTietue, TYPE tietue) {
        gridTietue.getChildren().clear();
        TextField[] edits = new TextField[tietue.getKenttia()];
        
        for (int i = 0, k = 1; k <= tietue.getKenttia(); k++, i++) {
            Label label = new Label(tietue.getKysymys(k));
            gridTietue.add(label, 0, i);
            
            TextField edit = new TextField();
            edit.setText(tietue.anna(k));
            edits[i] = edit;
            edit.setId("e"+k);
            gridTietue.add(edit, 1, i);
        }
        return edits;
    }
    
    
    /**
     * @param kentat teksikentät, joissa tiedot näytetään 
     * @param t tietue jonka tiedot näytetään
     */
    public static void naytaTietue(TextField[] kentat, Tietue t) {
        if (t == null) return;
        int i = 0;
        for (int k = 1; k <= t.getKenttia(); k++) {
            kentat[i].setText(t.anna(k));
            i++;
        }
    }
    
    
    /**
     * Näyttää listasta valitun jäsenen hiihdot
     */
    protected void naytaHiihdot() {
        gridHiihdot.clear();
        ArrayList<Hiihto> jasenenHiihdot = rekisteri.getJasenenHiihdot(tietueKohdalla.getID());
        for (int i = 0; i < jasenenHiihdot.size(); i++) {
                String[] rivi = jasenenHiihdot.get(i).getTiedot();
                gridHiihdot.add(jasenenHiihdot.get(i), rivi);
        }
        gridHiihdot.setOnMouseClicked( e -> { if ( e.getClickCount() > 1 ) muokkaaHiihtoa(); } );
    }

    
    /**
     * Muokataan valittuna olevaa hiihtosuoritusta.
     */
    private void muokkaaHiihtoa() {
        int r = gridHiihdot.getRowNr();
        if ( r < 0 ) return; // klikattu ehkä otsikkoriviä
        Hiihto h = gridHiihdot.getObject();
        if ( h == null ) return;
        //nt k = gridHiihdot.getColumnNr()+h.ekaKentta();
        try {
            h = TietueGUIController.kysyTietue(null, h.clone());
            if ( h == null ) return;
            rekisteri.korvaaTaiLisaa(h); 
            naytaHiihdot(); //tietueKohdalla 
            gridHiihdot.selectRow(r);  // järjestetään sama rivi takaisin valituksi
        } catch (CloneNotSupportedException  e) {
            //
        } catch (SailoException e) {
            Dialogs.showMessageDialog("Ongelmia lisäämisessä: " + e.getMessage());
        }  
    }
    
    
    /**
     * Poistetaan valittu hiihto reksiteristä
     * @return true jos poisto onnistui
     */
    private boolean poistaHiihto() {
        int r = gridHiihdot.getRowNr();
        if ( r < 0 ) return false; // klikattu ehkä otsikkoriviä
        Hiihto h = gridHiihdot.getObject();
        if ( h == null ) return false;
        
        if ( !Dialogs.showQuestionDialog("Poisto", "Poistetaanko hiihto", "Kyllä", "Ei") )
            return false;
        rekisteri.poistaHiihto(h);
        return true;
    }
    
    
    /**
     * Käsitellään jäseneen tullut muutos
     * @param edit muuttunut kenttä
     */
    private void kasitteleMuutosTietueeseen(TextField edit) {
        if (tietueKohdalla == null) return;
        int k = getFieldId(edit, tietueKohdalla.ekaKentta());
        String s = edit.getText();
        String virhe = null;
        //if (s.equals("")) return;
        virhe = tietueKohdalla.aseta(k,s);
        if (virhe == null) {
            Dialogs.setToolTipText(edit,"");
            edit.getStyleClass().removeAll("virhe");
            naytaVirhe(virhe);
        } else {
            Dialogs.setToolTipText(edit,virhe);
            edit.getStyleClass().add("virhe");
            naytaVirhe(virhe);
        }
    }
    
    
    /**
     * Palautetaan komponentin id:stä saatava luku
     * @param obj tutkittava komponentti
     * @param oletus mikä arvo jos id ei ole kunnollinen
     * @return komponentin id lukuna 
     */
    public static int getFieldId(Object obj, int oletus) {
        if ( !( obj instanceof Node)) return oletus;
        Node node = (Node)obj;
        return Mjonot.erotaInt(node.getId().substring(1),oletus);
    }
    
    
    /**
     * Näyttää virheilmoitukset
     * @param virhe joka näytetään
     */
    private void naytaVirhe(String virhe) {
        if ( virhe == null || virhe.isEmpty() ) {
            labelVirhe.setText("");
            labelVirhe.getStyleClass().removeAll("virhe");
            return;
        }
        labelVirhe.setText(virhe);
        labelVirhe.getStyleClass().add("virhe");
    }
    
    
    /**
     * Luodaan jäsenen kysymisdialogi ja palautetaan sama tietue muutettuna tai null
     * TODO: korjattava toimimaan
     * @param <TYPE> tietueen tyyppi
     * @param modalityStage mille ollaan modaalisia, null = sovellukselle
     * @param oletus mitä dataan näytetään oletuksena
     * @return null jos painetaan Cancel, muuten täytetty tietue
     */
    public static<TYPE extends Tietue> TYPE kysyTietue(Stage modalityStage, TYPE oletus) {
        return ModalController.<TYPE, TietueGUIController<TYPE>>showModal(
                    TietueGUIController.class.getResource("LisaaTietue.fxml"),
                    "Hiihtorekisteri",
                    modalityStage, oletus, ctrl -> ctrl.alusta()
                );
    }

    
    /**
     * @param <TYPE> tietueen tyyppi
     * @param modalityStage modaalisia, null = sovellukselle
     * @param oletus mitä näytetään oletuksena
     * @param r käytettävä rekisteri
     * @return valittu hiihto
     */
    public static<TYPE extends Tietue> TYPE kysyTietue(Stage modalityStage, TYPE oletus, Rekisteri r) {
        return ModalController.<TYPE, TietueGUIController<TYPE>>showModal(
                    TietueGUIController.class.getResource("Muokkaa.fxml"),
                    "Hiihtorekisteri",
                    modalityStage, oletus, 
                    ctrl -> ctrl.setRekisteri(r)
                );
    }
    
    
    /**
     * Asetetaan rekisteri
     * @param rek asetettava rekisteri
     */
    public void setRekisteri(Rekisteri rek) {
        rekisteri = rek;
        alusta();
        naytaTietue(kentat, tietueKohdalla);
        if (gridHiihdot != null) naytaHiihdot();
    }
}